@@ -22,7 +22,7 @@ KEYWORD = {
 
 SELECT, FROM, WHERE, GROUP_BY, ORDER_BY, LIMIT = KEYWORD.keys()
 USUAL_KEYS = [SELECT, WHERE, GROUP_BY, ORDER_BY, LIMIT]
-TO_LIST = lambda x: x if isinstance(x, list) else [x]
+TO_LIST = lambda x: x if isinstance(x, list) else [] if x is None else [x]
 
 
 class SQLObject:
@@ -277,7 +277,7 @@ class Function(Code):
                     return Cast(func, main_param)
             return param
         # ----------------------------------------
-        self.params = [set_func_types(p) for p in params]
+        self.params = [set_func_types(p) for p in params if p is not None]
         self.pattern = self.get_pattern()
         super().__init__()
 
@@ -376,6 +376,35 @@ class SubString(Function):
             return 'Substr({params})'
         return super().get_pattern()
 
+
+class Re(Function):
+    """
+    Extracts a substring that matches a pattern
+    """
+    inputs = [CHAR, CHAR, INT, INT]
+
+    def get_pattern(self) -> str:
+        if self.dialect == Dialect.POSTGRESQL:
+            return 'Substring({params})'
+        return 'Regexp_Substr({params})'
+    
+    @classmethod
+    def number_before(cls, string: str, start: int=None, end:int=None):
+        return cls(fr'(\d+)\s*{string}', start, end)
+
+    @classmethod
+    def number_after(cls, string: str, start: int=None, end:int=None):
+        return cls(fr'{string}\s*(\d+)', start, end)
+
+    @classmethod
+    def word_before(cls, string: str, start: int=None, end:int=None):
+        return cls(fr'(\w+)\s*{string}', start, end)
+
+    @classmethod
+    def word_after(cls, string: str, start: int=None, end:int=None):
+        return cls(fr'{string}\s*(\w+)', start, end)
+
+
 # ---- Numeric Functions: --------------------------------
 class Round(Function):
     """
@@ -644,7 +673,11 @@ def quoted(value) -> str:
     if isinstance(value, str):
         if re.search(r'\bor\b', value, re.IGNORECASE):
             raise PermissionError('Possible SQL injection attempt')
-        value = f"'{value}'"
+        return f"'{value}'"
+    elif isinstance(value, Select):
+        query: Select = value
+        query.break_lines = False
+        return f'({query})'
     return str(value)
 
 
@@ -701,10 +734,10 @@ class Where:
         return cls('IS NULL')
     
     @classmethod
-    def inside(cls, values):
+    def inside(cls, values, keyword: str='IN'):
         if isinstance(values, list):
             values = ','.join(quoted(v) for v in values)
-        return cls(f'IN ({values})')
+        return cls(f'{keyword} ({values})')
 
     @classmethod
     def formula(cls, formula: str):
@@ -774,6 +807,7 @@ class Not(Where):
 
 class Case:
     break_lines = True
+    quoted_result: bool = True
 
     def __init__(self, field: str):
         self.__conditions = {}
@@ -782,18 +816,13 @@ class Case:
         self.current_condition = None
         self.fields = []
 
-    def when(self, condition: Where, result=None):
+    def when(self, condition: Where):
         self.current_condition = condition
-        if result is None:            
-            return self
-        return self.then(result)
+        return self
     
     def then(self, result):
-        if isinstance(result, str):
-            if result.startswith('='):
-                result = result[1:]
-            else:
-                result = quoted(result)
+        if self.quoted_result:
+            result = quoted(result)
         self.__conditions[result] = self.current_condition
         return self
     
@@ -803,7 +832,11 @@ class Case:
         self.default = default
         return self
     
-    def format(self, name: str, field: str='') -> str:
+    def format(self, name: str, main:SQLObject, field: str='') -> str:
+        def put_alias(s: str) -> str:
+            is_quoted = re.search(r'[\'"]', s)
+            no_alias = (is_quoted or not main)
+            return s if no_alias else Field.format(s, main)
         TABULATION = '\t\t' if self.break_lines else ' '
         LINE_BREAK = '\n' if self.break_lines else ' '
         default = self.default
@@ -812,7 +845,7 @@ class Case:
         return 'CASE{brk}{cond}{df}{tab}END{alias}'.format(
             brk=LINE_BREAK,
             cond=LINE_BREAK.join(
-                f'{TABULATION}WHEN {field} {cond.content} THEN {res}'
+                f'{TABULATION}WHEN {put_alias(field)} {cond.content} THEN {put_alias(res)}'
                 for res, cond in self.__conditions.items()
             ),
             df=f'{LINE_BREAK}{TABULATION}ELSE {default}' if not default is None else '',
@@ -821,12 +854,12 @@ class Case:
         )
     
     def __str__(self):
-        return self.format('', self.field)
+        return self.format('', None, self.field)
 
     def add(self, name: str, main: SQLObject):
         main.values.setdefault(SELECT, []).append(
             self.format(
-                name, Field.format(self.field, main)
+                name, main, Field.format(self.field, main)
             )
         )
 
@@ -892,16 +925,41 @@ class If(Code, Frame):
         super().__init__()
 
     def format(self, name: str, main: SQLObject) -> str:
-        return '{func}({param}){over}'.format(
-            func=self.func_class.__name__,
-            param=Case(self.field).when(self.condition, f'={name}').else_value(0),
-            over=self.pattern
-        )
+        quoted_result = Case.quoted_result
+        Case.quoted_result = False
+        _case = Case(self.field).when(self.condition).then(name).else_value(0)
+        Case.quoted_result = quoted_result
+        return self.func_class(
+            _case.format('', main)
+        ).format(name, main)
     
     def get_pattern(self) -> str:
         return ''
 
 
+class Pivot:
+    where_method = Where.eq
+
+    def __init__(self, values: list, result: str, func_class: Function=Sum):
+        self.values = values
+        self.func_class = func_class
+        self.result = str(result)
+
+    def add(self, name: str, main: 'Select'):
+        partition_params = Partition.params
+        Partition.params = None
+        for value in self.values:
+            if isinstance(value, (tuple, list)):
+                value, label = value
+            else:
+                label = value
+            If(
+                name, self.func_class,
+                self.where_method(value)
+            ).As(label).add(self.result, main)
+        Partition.params = partition_params
+
+
 class Options:
     def __init__(self, **values):
         self.__children: dict = values
@@ -955,8 +1013,8 @@ class Range(Case):
         cls = self.__class__
         for label, value in sorted(values.items(), key=lambda item: item[1]):
             self.when(
-                Between(start, value).literal(), label
-            )
+                Between(start, value).literal()
+            ).then(label)
             start = cls.INC_FUNCTION(value)
 
 
@@ -1048,17 +1106,21 @@ class OrderBy(Clause):
 
 class Partition:
     params = None
+    content = None
 
     @classmethod
     def cls_to_str(cls, field: str) -> str:
         return f'PARTITION BY {field}'
     
     def __init__(self, content):
-        self.content = content
+        Partition.content = content
 
-    def add(self, name: str, main: SQLObject):
-        Partition.params = {name: Partition}  # <=== class variable!
-        self.content.add(name, main)
+    @classmethod
+    def add(cls, name: str, main: SQLObject):
+        cls.params = {name: Partition}
+        if cls.content:
+            cls.content.add(name, main)
+        cls.content = None
 
 
 class GroupBy(Clause):
@@ -1755,7 +1817,7 @@ class CypherParser(Parser):
 
     def prepare(self):
         self.REGEX['separator'] = re.compile(fr'({self.CHAR_SET}|->|<-|{self.KEYWORDS})')
-        self.REGEX['condition'] = re.compile(r'(\w+)(\s*[<>=]+\s*["\']*[\d+|\w+|\+\-\*\/]*["\']*)')
+        self.REGEX['condition'] = re.compile(r'([<>=]|IN|LIKE)', re.IGNORECASE)
         self.REGEX['alias_pos'] = re.compile(r'(\w+)[.](\w+)')
         self.join_type = JoinType.INNER
         self.TOKEN_METHODS = {
@@ -1768,7 +1830,7 @@ class CypherParser(Parser):
         self.aliases = {}
 
     def new_query(self, token: str, join_type = JoinType.INNER, alias: str=''):
-        token, *group_fields = token.split('@')
+        token, *more = re.split(r"([|@])", token)
         if not token.isidentifier():
             return
         table_name = f'{token} {alias}' if alias else token
@@ -1777,7 +1839,10 @@ class CypherParser(Parser):
             alias = query.alias
         self.queries.append(query)
         self.aliases[alias] = query
-        FieldList(group_fields, [Field, GroupBy]).add('', query)
+        if more:
+            for sep, expr in zip(more[::2], more[1::2]):
+                class_type = {'@': GroupBy, '|': Partition}[sep]
+                self.add_field(expr, [class_type])
         query.join_type = join_type
 
     def add_where(self, token: str):
@@ -1786,16 +1851,14 @@ class CypherParser(Parser):
             alias, field, *condition = elements
             query = self.aliases[alias]
         else:
-            field, *condition = [
-                t for t in self.REGEX['condition'].split(token) if t
-            ]
+            field, *condition = self.REGEX['condition'].split(token)
             query = self.queries[-1]
         Where(' '.join(condition)).add(field, query)
     
     def add_order(self, token: str):
-        self.add_field(token, sorted=True)
+        self.add_field(token, [OrderBy])
 
-    def add_field(self, token: str, sorted: bool = False):
+    def add_field(self, token: str, class_types: list = None):
         if token in self.TOKEN_METHODS:
             return
         if '*' in token:
@@ -1823,7 +1886,7 @@ class CypherParser(Parser):
             is_count = function == 'count'
             if alias or is_count:
                 field, alias = alias, field
-            extra_classes = [OrderBy] if sorted else []
+            extra_classes = class_types or []
             if group:
                 if not field:
                     field = group
@@ -1833,10 +1896,10 @@ class CypherParser(Parser):
             if function:                
                 if is_count and not field:
                     field = self.queries[-1].key_field or 'id'
-                class_type = FUNCTION_CLASS.get(function)
-                if not class_type:
+                func_class = FUNCTION_CLASS.get(function)
+                if not func_class:
                     raise ValueError(f'Unknown function `{function}`.')
-                class_list = [ class_type().As(alias, extra_classes) ]
+                class_list = [ func_class().As(alias, extra_classes) ]
             elif alias:
                 class_list = [NamedField(alias)] + extra_classes
             else:
@@ -1855,6 +1918,14 @@ class CypherParser(Parser):
         self.new_query(token, JoinType.RIGHT)
 
     def add_foreign_key(self, token: str, pk_field: str=''):
+        def extract_field(query: Select, pos: int) -> str:
+            fields = [
+                fld for fld in query.values[SELECT]
+                if fld not in query.values.get(GROUP_BY, [])
+            ]
+            result  = fields[pos].split('.')[-1]
+            query.delete(result, [SELECT], exact=True)
+            return result
         curr, last = [self.queries[i] for i in (-1, -2)]
         if not pk_field:
             if last.key_field:
@@ -1862,8 +1933,7 @@ class CypherParser(Parser):
             else:
                 if not last.values.get(SELECT):
                     raise IndexError(f'Primary Key not found for {last.table_name}.')
-                pk_field = last.values[SELECT][-1].split('.')[-1]
-                last.delete(pk_field, [SELECT], exact=True)
+                pk_field = extract_field(last, -1)
         if '{}' in token:
             foreign_fld = token.format(
                 last.table_name.lower()
@@ -1873,12 +1943,7 @@ class CypherParser(Parser):
         else:
             if not curr.values.get(SELECT):
                 raise IndexError(f'Foreign Key not found for {curr.table_name}.')
-            fields = [
-                fld for fld in curr.values[SELECT]
-                if fld not in curr.values.get(GROUP_BY, [])
-            ]
-            foreign_fld = fields[0].split('.')[-1]
-            curr.delete(foreign_fld, [SELECT], exact=True)
+            foreign_fld = extract_field(curr, 0)
             if curr.join_type == JoinType.RIGHT:
                 pk_field, foreign_fld = foreign_fld, pk_field
         if curr.join_type == JoinType.RIGHT:
@@ -2192,28 +2257,46 @@ class Select(SQLObject):
             rule.apply(self)
 
     def add_fields(self, fields: list, class_types=None):
-        if not class_types:
-            class_types = []
-        class_types += [Field]
+        class_types = TO_LIST(class_types)
+        has_partition = any(isinstance(cls, Partition) for cls in class_types)
+        if not has_partition:
+            class_types += [Field]
         FieldList(fields, class_types).add('', self)
 
     def translate_to(self, language: QueryLanguage) -> str:
         return language(self).convert()
 
 
-class SelectIN(Select):
+# -------------------------------------------------------
+class SubSelect(Select):
     condition_class = Where
+    SUBQUERY_KEYWORD = 'IN'
 
     def add(self, name: str, main: SQLObject):
         self.break_lines = False
-        self.condition_class.inside(self).add(name, main)
+        self.condition_class.inside(
+            self, self.SUBQUERY_KEYWORD
+        ).add(name, main)
+
+
+class SelectIN(SubSelect):
+    ...
 
-SubSelect = SelectIN
 
 class NotSelectIN(SelectIN):
     condition_class = Not
 
 
+class SelectExists(SubSelect):
+    SUBQUERY_KEYWORD = 'EXISTS'
+
+
+class NotSelecExists(SelectExists):
+    condition_class = Not
+
+# -------------------------------------------------------
+
+
 class CTE(Select):
     prefix = ''
     show_query = True
@@ -2301,10 +2384,124 @@ class Recursive(CTE):
         return self
 
 
+class CTENode:
+    TEMPLATE_FIELD_FUNC = lambda t: t[:3].lower() + '_id'
+
+    def __init__(self, descr: str='', pos: int=-1, parent: 'CTENode' = None):
+        self.description = descr
+        self.pos = pos
+        self.parent = None
+        self.children = []
+        if parent:
+            parent.add(self)
+        self.content = ''
+        self.expected_char = ''
+        self.sql_flag = {}
+
+    def add(self, child: 'CTENode'):
+        self.children.append(child)
+        child.parent = self
+
+    def is_sql(self) -> bool:
+        return any(self.sql_flag.values())
+
+    def has_join(self) -> bool:
+        child: 'CTENode'
+        for child in self.children:
+            if 'JOIN' in child.sql_flag:
+                return True
+        return False
+
+    def tree(self, function: callable):
+        for child in self.children:
+            child.tree(function)
+        function(self)
+
+    @classmethod
+    def create(cls, txt: str, template: str='') -> 'CTENode':
+        PAIR = {'(': ')', '[': ']'}
+        def pattern() -> str:
+            REGEX_OPENING = ''.join( fr"\{char}" for char in PAIR.keys() )
+            REGEX_CLOSING = ''.join( fr"\{char}" for char in PAIR.values() )
+            return fr'(\w+)\s*[{REGEX_OPENING}]|[{REGEX_CLOSING}]'
+        # -------------------------------------------------------------------
+        def get_sql_children(node: 'CTENode', x: int):
+            if not node.parent and node.description == '':
+                node.description = re.sub(r"\s+", ' ', txt[:node.pos]).strip()
+            found = re.search(r'[)]\s*AS\s+(\w+)', txt[x-1:], re.IGNORECASE)
+            if found:
+                node.description = found.group(1)
+            REGEX_UNION = r'UNION\s+ALL|\bunion\b|\bUNION\b|union\s+all'
+            for sub in re.split(REGEX_UNION, node.content):
+                cls('', -1, node).content = re.sub(r"\s+", ' ', sub).strip()
+            if node.sql_flag['JOIN'] and node.parent:
+                REGEX_FIELD = r"\s+\w+[.]\w+\s*"
+                REGEX_JOIN = fr"\bON\b({REGEX_FIELD}={REGEX_FIELD})"
+                node = node.parent
+                arr = [child.description for child in node.children]
+                params = {
+                    'fld': re.findall(r'select\s+(.*)\s+from\b\s*[(]', txt[:x], re.IGNORECASE)[0],
+                    't1': arr[0], 't2': arr[1],
+                    'expr': re.findall(REGEX_JOIN, txt[x:], re.IGNORECASE)[0].strip(),
+                }
+                node.content = 'SELECT {fld} FROM {t1} {t1} JOIN {t2} {t2} ON {expr}'.format(**params)
+        # -------------------------------------------------------------------
+        if template:
+            for name in re.findall(r'[#](\w+)', txt):
+                old = f"#{name}"
+                new = template.format(
+                    t=name, f=cls.TEMPLATE_FIELD_FUNC(name)
+                )
+                txt = txt.replace(old, new)
+        node: 'CTENode' = None
+        root: 'CTENode' = None
+        main: 'CTENode' = None
+        orphans: list = []
+        ignore: int = 0
+        for found in re.finditer(pattern(), txt):
+            i = found.end()
+            if found.group() in PAIR.values():
+                if not node or found.group() != node.expected_char:
+                    continue
+                if ignore:
+                    ignore -= 1
+                    continue
+                if not node.has_join():
+                    node.content = re.sub(r'\s+', ' ', txt[node.pos: i-1])
+                if not node.parent:
+                    orphans.append(node)
+                    if len(orphans) > 1:
+                        if not main:
+                            main = cls('main')
+                            main.content = txt[i:].strip()
+                        for lost in orphans:
+                            main.add(lost)
+                        orphans = []                            
+                        root = main
+                    elif node == root and node.content == '':
+                        node.content = txt[i:].strip()
+                if node.is_sql():
+                    get_sql_children(node, i)
+                node = node.parent
+            else:
+                name = found.group(1)
+                sql_flag = {key: name.upper() == key for key in ('FROM', 'JOIN')}
+                if any(sql_flag.values()):
+                    name = ''
+                elif txt[i-1] == '(':
+                    if not node or node.is_sql():
+                        ignore += 1
+                    continue
+                node = cls( name, i, node )
+                if not root:
+                    root = node
+                node.sql_flag = sql_flag
+                node.expected_char = PAIR[txt[i-1]]
+        return root
+
+
+
 class CTEFactory:
-    MAIN_TAG = '__main__'
-    TEMPLATE_FIELD_FUNC = lambda t: t.lower()[:3] + '_id'
-    ALL_FIELDS = 'all_fields'
 
     def __init__(self, txt: str, template: str = ''):
         """
@@ -2319,61 +2516,61 @@ class CTEFactory:
         `cte_name`[
             Table1(field, `function$`field`:alias`, `group@`) <- Table2(field)
         ]
+
+        template (optional):
+        ---
+        * `{t} = Table name`
+        * `{f} = Field name` (runs CTEFactory.TEMPLATE_FIELD_FUNC)
+
+        > Example: txt="#AAA #BBB", template="SELECT * FROM {t} WHERE {f} = 217"
+        ...results:
+        ```
+            SELECT * FROM AAA WHERE aaa_id = 217
+            UNION
+            SELECT * FROM BBB WHERE bbb_id = 217
+        ```
         """
-        NEGATIVE_MARK = '{last'
-        self.names = {}
-        if template:
-            for table in re.findall(r'[#](\w+)', txt):
-                txt = txt.replace( f'#{table}', template.format(
-                    t=table, f=CTEFactory.TEMPLATE_FIELD_FUNC(table)
-                ) )
-        txt = self.replace_wildcards(txt)
         self.cte_list = []
         self.main = None
-        script_list = txt.split(']')
-        is_negative = False
-        for i, script in enumerate(script_list, 1):            
-            if '{' in script:
-                is_negative = NEGATIVE_MARK in script
-                script = script.format(**self.names) 
-            if i == len(script_list) and is_negative:
-                break
-            self.build_ctes(script)
-            self.fill_names()
-        if not self.cte_list:
-            return
-        if is_negative:
-            self.main = detect(script)
-        elif not self.main:
-            self.main = Select(self.cte_list[-1].table_name)
-            self.main.break_lines = False
+        node = CTENode.create(txt, template)
+        node.tree(self.build_ctes)
 
-    def build_ctes(self, script: str):
-        alias, *body = script.split('[')
-        if body:
-            script = ''.join(body)
-        if not re.sub( r'\s+', '', script):
+    def build_ctes(self, node: CTENode):
+        # ================================================
+        def generic_query(node: CTENode) -> Select:
+            result = Select(node.description)
+            result.break_lines = False
+            return result
+        # -----------------------------------------------
+        try:
+            query = detect(node.content)
+        except:
+            query = None
+        # -----------------------------------------------
+        if not node.description:
+            node.content = query
             return
-        if parser_class(script) == CypherParser:
-            query_list = Select.parse(script, CypherParser)
-            if not body:
-                alias = '_'.join(query.table_name for query in query_list)
-            related_tables = any([
-                query.join_type.value for query in query_list
-            ])
-            if related_tables:
-                query_list = [ join_queries(query_list) ]
-            self.cte_list += [CTE(alias, query_list)]
-            return 
-        summary = self.extract_subqueries(script)
-        self.main = detect( summary.pop(self.MAIN_TAG) )
-        self.cte_list += [
-            CTE(alias, [
-                Select.parse(query)[0]
-                for query in elements
-            ])
-            for alias, elements in summary.items()
-        ]
+        elif node.children:
+            query_list = []
+            child: CTENode
+            for child in node.children:
+                if child.description:
+                    query_list.append( detect(f'SELECT * FROM {child.description}') )
+                else:
+                    query_list.append(child.content)
+        else:
+            query_list = [query]
+        # -----------------------------------------------
+        if not node.parent: # node == node.root
+            if not query:
+                self.main = generic_query(node)
+            elif node.expected_char == ']' or node.has_join():
+                query_list = [query]
+                self.main = generic_query(node)
+        # -----------------------------------------------
+        if all(query_list):
+            self.cte_list.append( CTE(node.description, query_list) )
+        # ================================================
 
     def __str__(self):
         if not self.main:
@@ -2383,108 +2580,6 @@ class CTEFactory:
         result = ',\n'.join(lines) + '\n' + str(self.main)
         CTE.show_query = True
         return result
-    
-    def fill_names(self):
-        def get_field_list(query_list) -> list:
-            field_list = []
-            for query in query_list:
-                for field in query.values.get(SELECT, []):
-                    new_item = re.split(
-                        r'\bas\b|\bAS\b|[.]', field
-                    )[-1].strip()
-                    if new_item not in field_list:
-                        field_list.append(new_item)
-            return field_list
-        query_list = []
-        cte: CTE
-        for i, cte in enumerate(self.cte_list):
-            name = f'tag{i+1}'
-            if name in self.names and self.names[name].startswith('{'):
-                self.names[name] = cte.table_name
-            negative_pos = abs( len(self.cte_list) - i )
-            name = f'last{negative_pos}'
-            if name in self.names:
-                self.names[name] = cte.table_name
-            query_list += cte.query_list
-        if self.ALL_FIELDS in self.names:
-            self.names[self.ALL_FIELDS] = ','.join( get_field_list(query_list) )
-    
-    def replace_wildcards(self, txt: str) -> str:
-        # ----------------------------------------------------
-        def has_parentheses(patch: str) -> bool:
-            return re.sub(r'\s+', '', patch).startswith('(')
-        # ----------------------------------------------------
-        result = ''
-        end = 0
-        for found in re.finditer(r'(\[[-]*\d+\])|([*][*])', txt):
-            start = found.start()
-            content = found.group()
-            if content == '**':
-                tag_id = self.ALL_FIELDS
-            else:
-                value = int( content[1:-1] )
-                if value < 0:
-                    prefix = 'last'
-                    value = -value
-                else:
-                    prefix = 'tag'
-                tag_id = f'{prefix}{value}'
-            mark = '{' + tag_id + '}'
-            if tag_id not in self.names:
-                self.names[tag_id] = mark
-            result += '{}{}'.format(
-                txt[end:start], mark
-            )
-            end = found.end()
-            if not has_parentheses(txt[end:]):
-                result += '()'
-        if not result:
-            return txt
-        result += txt[end:]
-        return result
-
-    @classmethod
-    def extract_subqueries(cls, txt: str) -> dict:        
-        result = {}
-        # ---------------------------------------------------
-        def clean_subquery(source: list) -> str:
-            while source:
-                if source[0].upper() == 'SELECT':
-                    break
-                word = source.pop(0)
-                if word.upper() in ('FROM', 'JOIN'):
-                    result[cls.MAIN_TAG] += f' {word}'
-            return ' '.join(source)
-        def balanced_parentheses(expr: str) -> bool:
-            return expr.count('(') == expr.count(')')
-        # ---------------------------------------------------
-        for found in re.finditer(r'(FROM|JOIN)\s*[(]\s*SELECT', txt, re.IGNORECASE):
-            start = found.start()
-            alias = ''
-            pos = start
-            while not alias:
-                found = re.search(r'[)]\s*AS\s+\w+', txt[pos:], re.IGNORECASE)
-                if not found:
-                    break
-                end = found.end() + pos
-                last = end
-                if balanced_parentheses(txt[start: end]):
-                    pos += found.start()
-                    alias = re.findall(r'\s*(\w+)$', txt[pos: end])[0]
-                    end = pos
-                pos = end
-            if not result:
-                result[cls.MAIN_TAG] = txt[:start]
-            query_list = [
-                clean_subquery( expr.split() )
-                for expr in  re.split(
-                    r'\bUNION\b', txt[start: end], flags=re.IGNORECASE
-                )
-            ]
-            result[cls.MAIN_TAG] += f' {alias} {alias}'
-            result[alias] = query_list
-        result[cls.MAIN_TAG] += txt[last:]
-        return result
 
 
 # ----- Rules -----
@@ -2523,6 +2618,32 @@ class RuleAutoField(Rule):
             s2 = set(target.values[ORDER_BY])
             target.values.setdefault(SELECT, []).extend( list(s2-s1) )
 
+class RuleCalcWithColumn(Rule):
+    @classmethod
+    def apply(cls, target: Select):
+        conditions = target.values[WHERE]
+        REGEX_ALPHA = r'[A-Za-z]+'
+        REGEX_FIELD = fr'({REGEX_ALPHA}[.])*({REGEX_ALPHA})'
+        REGEX_MATH_OP = r'([\+\-\*\/])'
+        REGEX_NUMBER = r'(\d+[.]\d+|\d+)'
+        REGEX_COMPARE = r'([><=]+)'
+        INVERSE_OP = {'+': '-', '-': '+', '*': '/', '/': '*'}
+        pattern = re.compile( r'\s*'.join([
+            REGEX_FIELD, REGEX_MATH_OP, REGEX_NUMBER, REGEX_COMPARE, REGEX_NUMBER
+        ]) )
+        count = 0
+        for i, cond in enumerate(conditions):
+            found = pattern.findall(cond)
+            if not found:
+                continue
+            alias, field, op, num1, comp, num2 = found[-1]
+            expr = f"{num2} {INVERSE_OP[op]} {num1}"
+            conditions[i] = "{} {} {}".format(
+                alias+field,
+                comp,
+                eval(expr)
+            )
+
 
 class RuleLogicalOp(Rule):
     REVERSE = {">=": "<", "<=": ">", "=": "<>"}
@@ -2583,7 +2704,7 @@ class RuleReplaceJoinBySubselect(Rule):
             if keep_join:
                 query.add(fk_field, main)
                 continue
-            query.__class__ = SubSelect
+            query.__class__ = SelectIN
             Field.add(primary_k, query)
             query.add(fk_field, main)
             modified = True
@@ -2634,21 +2755,46 @@ def detect(text: str, join_method = join_queries, format: str='') -> Select | li
     return result
 # ===========================================================================================//
 
+
 if __name__ == "__main__":
-    TABLE_NAME = 'Sales s'
-    DATE_FIELD = 'ref_date'
-    print('▒'*50)
-    q1 = Select(
-        TABLE_NAME, 
-        ref_year=Year(DATE_FIELD),
-        price=Sum().As('total').over(ref_year=Partition)
+    cte = CTEFactory(
+    """
+            SELECT u001.name, agg_sales.total
+            FROM (
+                SELECT * FROM Users u
+                WHERE u.status = 'active'
+            ) AS u001
+            JOIN (
+                SELECT s.user_id, Sum(s.value) as total
+                FROM Sales s
+                GROUP BY s.user_id
+            )
+            As agg_sales
+            ON u001.id = agg_sales.user_id
+    """
+    # ===========================================================================================
+    # """
+    #     People[
+    #         #Employee  #Customer  #Supllier
+    #     ]
+
+    #     People(qt_sold, person_name, person_type, ref_year) <- Goal(ref_year, target)
+    # """
+    # ,template='''
+    #     Sales_by_{t}[
+    #         Sales(year$ref_date:ref_year@, sum$quantity:qt_sold,
+    #         {f}) -> {t}(id, name:person_name@, "{t[0]}":person_type)
+    #     ]
+    # '''
+    # ===========================================================================================
+    # '''
+    # u001[
+    #     User(name ?status="active", id)
+    # ]
+    
+    # agg_sales[
+    #     Sales(user_id@, sum$value:total)
+    # ]
+    # '''
     )
-    print(q1)
-    print('='*50)
-    q2 = Select(
-        TABLE_NAME, 
-        ref_year=Partition( Year(DATE_FIELD) ),
-        total=Sum('price'),
-    )
-    print(q2)
-    print('▒'*50)
+    print(cte)
\ No newline at end of file
